// txtsql - kometbomb - GPL

#include <stdio.h>
#include <sqlite3.h>
#include <string.h>

#define DB_NAME "txtsql.temp.lqstxt"
#define LINE_LENGTH 1000

static sqlite3 *db;


static int callback(void *NotUsed, int argc, char **argv, char **azColName){
	static int row_count;
	if ((int)NotUsed==1) {
		row_count=0;
		return 0;	
	}
	
  int i;
  
  if (row_count==0) {
	 	for(i=0; i<argc; i++){
		 	if (i) printf(" | ");
		 	if (strcmp(azColName[i],"row")==0)
	  		printf("%4s", azColName[i] ? azColName[i] : "NULL");
	  	else
    		printf("%s", azColName[i] ? azColName[i] : "NULL");		
	 	} 
	 	
	 	printf("\n");
	 	
	 	for(i=0; i<72; i++){
		 	putc('-',stdout);	
	 	}
	 	
	 	printf("\n");
  }
  
  for(i=0; i<argc; i++){
	  if (i) printf(" | ");
	  if (strcmp(azColName[i],"row")==0)
	  	printf("%4s", argv[i] ? argv[i] : "NULL");
	  else
    	printf("%s", argv[i] ? argv[i] : "NULL");
  }
  printf("\n");
  
  row_count++;
  
  return 0;
}

static void showerror(char *msg) {
	fprintf(stderr, "SQL error: %s\n", msg);
	sqlite3_free(msg);	
}

int newfile(char *name) {
	char *sql=sqlite3_mprintf("create table `%q` (row integer primary key autoincrement,data text)",name);
	int rc;
	char *zErrMsg=NULL;
	  	  
	rc = sqlite3_exec(db, sql , NULL, 0, &zErrMsg);
	sqlite3_free(sql);
	
	if (rc!=SQLITE_OK) {
		showerror(zErrMsg);
		return 0;
	}
	
	return 1;
}

void save(char *table) {
	char *sql=sqlite3_mprintf("select * from `%q`",table);
	char **result=NULL;
	int rows=0,cols=0,rc;
  char *zErrMsg=NULL;
  
  rc = sqlite3_get_table(db, sql , &result, &rows, &cols, &zErrMsg);
  sqlite3_free(sql);
  
  if( rc!=SQLITE_OK ) {
    showerror(zErrMsg);
    return;
  } 
  
	FILE *f=fopen(table,"w");
	if (f) {
		int i;
		for (i=0;i<rows;i++) {
			if (strcmp(result[0],"row")!=0) 
				fprintf(f,"%s\t",result[cols+i*cols]);
			int c;
			for(c=1;c<cols;c++)
				fprintf(f,"%s",result[cols+i*cols+c]);
				
			fprintf(f,"\n");
		}
		
		sqlite3_free_table(result);
		fclose(f);	
	} else {
		fprintf(stderr,"Can't open %s\n",table);	
	}
	
}

void load(char *file) {
	char *zErrMsg = 0;
  int rc;
  
  FILE *f=fopen(file,"r");
  if (f) {
	  
	  if( !newfile(file) ) {
	    return;
	  } else {
		  rc = sqlite3_exec(db, "begin" , NULL, 0, &zErrMsg);
		  
		  if( rc!=SQLITE_OK ) {
		    showerror(zErrMsg);
		  } else
		  while (!feof(f)) {
			 	char line[LINE_LENGTH+1]; 
			 	if (fgets(line,LINE_LENGTH,f)) {
				 	char *c=line+strlen(line)-1;
				 	
				 	while (c>=line && (*c=='\r'||*c=='\n'))
				 		*(c--)='\0';
				 	
			 		char *sql=sqlite3_mprintf("insert into `%q` (data) values ('%q')",file,line);
  
				  rc = sqlite3_exec(db, sql , NULL, 0, &zErrMsg);
				  sqlite3_free(sql);
				  
				  if( rc!=SQLITE_OK ) {
				    showerror(zErrMsg);
				    break;
				  } 
			 	}
		  }
		  rc = sqlite3_exec(db, "commit" , NULL, 0, &zErrMsg);
		  
		  if( rc!=SQLITE_OK ) {
		    showerror(zErrMsg);
		  } 
	  }
  	fclose(f);
	} else {
  	fprintf(stderr,"Can't open %s\n",file);	
	}	
}

void help() {
	printf("Commands:\n  help\tThis help\n  new <table>\t CREATE TABLE (row int,data text)\n"
	"  load <file>\tLoads a text file in a table created with 'new'\n"
	"  save <table>\tSaves the table as a text file, cells separated with tabs\n"
	"  quit\tQuits txtsql\n");
}

int main(int argc, char **argv){
  char *zErrMsg = 0;
  int rc;

  if( argc<2 ){
    fprintf(stderr, "Usage: %s <file> [file...]\n", argv[0]);
    return 1;
  }
  rc = sqlite3_open(DB_NAME, &db);
  if( rc ){
    fprintf(stderr, "Can't open database: %s\n", sqlite3_errmsg(db));
    sqlite3_close(db);
    return 1;
  }
  
  int i;
  for (i=1;i<argc;i++) {
	  load(argv[i]);
  }
  
  printf("-- txtsql\n-- Type 'help' for help\n\n");
  
  while (1) {
	  printf("sql> ");
  	char line[2000],cmd[512],param[512];
	  if (!fgets(line,1999,stdin)) break;
	  
	  if (strnicmp(line,"quit",4)==0) break;
	  if (strnicmp(line,"help",4)==0) {
		  help(); 
		  continue;
	  }else if (sscanf(line,"%512s%*[ ]`%512[^`]`",cmd,param)==2 
	  	|| sscanf(line,"%512s%*[ ]%512s",cmd,param)==2) {
		  if (stricmp(cmd,"load")==0) {
			  load(param);
			  continue;
		  } else if (stricmp(cmd,"save")==0) {
			  save(param);
			  continue;
		  } else if (stricmp(cmd,"new")==0) {
			  newfile(param);
			  continue;
		  } 
		  
	  } 
  
	  
	  callback((void*)1,0,NULL,NULL);
	  rc = sqlite3_exec(db, line, callback, 0, &zErrMsg);
	  
	  if( rc!=SQLITE_OK ){
	    showerror(zErrMsg);
	  } else {
		 	printf("%d rows affected.\n",sqlite3_changes(db)); 
	  }
	
	}
  	
	sqlite3_close(db);
  
  unlink(DB_NAME);
  
  return 0;
}
